module assumptions
open bytes
open crypto_interface

(* -------------- Global Types -------------- *)
type error = eq bytes
type result = eq bytes

(* -------------- Global Functions -------------- *)
val choice : x:eq bytes -> y:eq bytes -> v:bytes{| (L v = L x) && (R v = R y) |}

(* -------------- Global Assumptions -------------- *)
(* A single ciphertext only corresponds to one plaintext *)
assume forall (mL1:bytes) (mL2:bytes) (mR1:bytes) (mR2:bytes) (c1:bytes) (c2:bytes).( 
			((Encryptedboth mL1 mR1 c1 c2)  && (Encryptedboth mL2 mR2 c1 c2)) 
			=> ((mL1 = mL2) && (mR1 = mR2)) 
			)			
			
(* Voter A only vote once. *)
assume forall (c1:bytes) (c2:bytes).( ((FromA c1) && (FromA c2)) => (c1 = c2) )

(* Voter B only vote once. *)
assume forall (c1:bytes) (c2:bytes).( ((FromB c1) && (FromB c2)) => (c1 = c2) )

(* -------------- Trash -------------- *)

(* -------------- Old Global Assumptions -------------- *)
(*
assume forall (m1:bytes) (m2:bytes) (c1:bytes) (c2:bytes). (
            ((FromOboth c1 c2) && (Encryptedboth m1 m2 c1 c2)) => (m1 = m2) )
			
assume forall (m:bytes) (ca:bytes) (cb:bytes) (co:bytes). (  
     ( (FromA ca) && (Valid ca) && (FromB cb) && (Valid cb) &&
     (Encrypted m co) && (Valid co) && (neq ca co) && (neq cb co) )  
     => (FromO co)
     )
	 
(* A valid vote is always coming from A, B or O. *)			
(* assume forall (c:bytes).( (Valid c) => ((FromA c) || (FromB c) || (FromO c)) ) *)
(* Note : No longer necessary because of the stuff contained in unseal_ref_val *)
(* Note : We can remove FromA and FromB from seal/unseal and FromO => equality if we keep it. *)

*)