(* Copyright (c) Microsoft Corporation.  All rights reserved.  *)

module bytes

logic data type bytes = (* improve to byte array *)
  | Utf8 : string -> bytes
  | Seq : int -> bytes

logic val Seqn: bytes -> int 
logic val SeqnInv:int -> bytes
assume SeqnInjective: forall a. SeqnInv (Seqn a) = a

(* logic val Utf8: string -> bytes  *)
val utf8:  s:string -> b:bytes { b = (Utf8 s) }
val iutf8: b:bytes -> s:string { b = (Utf8 s) }

(* (\* bad *\)  *)
(* assume forall (x:string) (y:string). (((Utf8 x)=(Utf8 y)) => x = y ) *)

val base64: bytes -> string
val ibase64: string -> bytes

type nat = n:int { 0 <= n }
logic val Length: bytes -> int 
logic val Concat: bytes -> bytes -> bytes 
logic val Xor : bytes -> bytes -> bytes
assume forall (x:bytes) (y:bytes). ((Length(Concat x y)) = ((Length x) + Length(y)))

type fbytes (i:int) = b:bytes{Length b = i}
logic val maxseq : int (* clean this up once we move to byte array *)
val toBytes: i:int -> b:fbytes 16{Seqn b = i}
(* val toBytes: i:int{i < maxseq} -> b:fbytes 16{Seqn i = b} *)


val length: b:bytes -> l:int {Length(b) = l}
val zeroCreate: l:int -> b:bytes {Length(b) = l}

val sub: b:bytes -> i:int -> l:int -> s:bytes {Length(s) = l}
val concat: b1:bytes -> b2:bytes -> b:bytes{b = Concat b1 b2}
val split: l:int -> b:bytes -> b1:bytes * b2:bytes 
  { Length b2 = l && b = Concat b1 b2 }
val truncate: l:int -> bytes -> b:bytes { Length(b) <= l }
val randomBytes: l:int -> b:bytes {Length(b) = l && |L b = R b|}

(* concatenation is injective when using fixed lengths *)
assume ConcatLeft: forall b1 b2 c1 c2. 
  (Concat b1 b2 = Concat c1 c2 && Length b1 = Length c1) => (b1=c1 && b2=c2)   
assume ConcatRight: forall b1 b2 c1 c2. 
  (Concat b1 b2 = Concat c1 c2 && Length b2 = Length c2) => (b1=c1 && b2=c2)   

(* // assume !b1,b2,c1,c2.  *)
(* //  (b1|b2 = c1|c2 && Length(b2)=Length(c2)) => (b1=c1 && b2=c2)    *)
(* // assume !c,c1,c2,l1,l2. *)
(* //  (c = c1|c2 && Length(c) = l1 + l2 &&  Length(c2) = l2 => Length(c1) = l1) *)

(*
function val Pad: (int * bytes) -> bytes
assume !l,x. Length(Pad(l,x)) = l

val pad:   l:int -> b:bytes (* {Length(b) < l} *) -> p:bytes {p = Pad(l,b)} 
val unpad: l:int -> p:bytes -> b:bytes {p = Pad(l,b)} 
(* unpad could return an option *)
*) 

(*
// marshalling for lengths 
function val Int2Bytes: int -> bytes
assume !i:int. Length(Int2Bytes(i)) = 2 
assume !i0:int, i1:int. Int2Bytes(i0)=Int2Bytes(i1) => i0=i1
val int2bytes: i:int -> b:bytes {b = Int2Bytes(i)} 
val bytes2int: b:bytes {Length(b)=2} -> i:int { b = Int2Bytes(i)}

val int_to_bytes: i:int -> b:bytes {b = Int2Bytes(i)} 
val bytes_to_int: b:bytes {Length(b)=2} -> i:int { b = Int2Bytes(i)}

(* 
ask
 !b0:bytes, b1:bytes, b0':bytes, b1':bytes. 
 Int2Bytes(Length(b0)) | b0 | b1 = Int2Bytes(Length(b0')) | b0' | b1' =>
 (b0=b0' && b1=b1') *)

val list_length: 'a list -> int

val siglength: int


val randomBytes: l:int -> b:bytes {Length(b) = l}


/// RPC-specific 

val send: bytes -> unit
val recv: (bytes-> unit) -> unit
*)
